@@ -358,6 +358,24 @@ static struct RequestTest {
 	  }
 	},
 
+	{ "NUL in header name", "760832",
+	  "GET / HTTP/1.1\r\nHost\x00: example.com\r\n", 36,
+	  SOUP_STATUS_OK,
+	  "GET", "/", SOUP_HTTP_1_1,
+	  { { "Host", "example.com" },
+	    { NULL }
+	  }
+	},
+
+	{ "NUL in header value", "760832",
+	  "GET / HTTP/1.1\r\nHost: example\x00" "com\r\n", 35,
+	  SOUP_STATUS_OK,
+	  "GET", "/", SOUP_HTTP_1_1,
+	  { { "Host", "examplecom" },
+	    { NULL }
+	  }
+	},
+
 	/************************/
 	/*** INVALID REQUESTS ***/
 	/************************/
@@ -418,20 +436,6 @@ static struct RequestTest {
 	  { { NULL } }
 	},
 
-	{ "NUL in header name", "666316",
-	  "GET / HTTP/1.1\r\n\x00: silly\r\n", 37,
-	  SOUP_STATUS_BAD_REQUEST,
-	  NULL, NULL, -1,
-	  { { NULL } }
-	},
-
-	{ "NUL in header value", NULL,
-	  "GET / HTTP/1.1\r\nHost: example\x00com\r\n", 37,
-	  SOUP_STATUS_BAD_REQUEST,
-	  NULL, NULL, -1,
-	  { { NULL } }
-	},
-
 	{ "No terminating CRLF", NULL,
 	  "GET / HTTP/1.1\r\nHost: example.com", -1,
 	  SOUP_STATUS_BAD_REQUEST,
@@ -608,6 +612,22 @@ static struct ResponseTest {
 	    { NULL } }
 	},
 
+	{ "NUL in header name", "760832",
+	  "HTTP/1.1 200 OK\r\nF\x00oo: bar\r\n", 28,
+	  SOUP_HTTP_1_1, SOUP_STATUS_OK, "OK",
+	  { { "Foo", "bar" },
+	    { NULL }
+	  }
+	},
+
+	{ "NUL in header value", "760832",
+	  "HTTP/1.1 200 OK\r\nFoo: b\x00" "ar\r\n", 28,
+	  SOUP_HTTP_1_1, SOUP_STATUS_OK, "OK",
+	  { { "Foo", "bar" },
+	    { NULL }
+	  }
+	},
+
 	/********************************/
 	/*** VALID CONTINUE RESPONSES ***/
 	/********************************/
@@ -702,18 +722,6 @@ static struct ResponseTest {
 	  { { NULL } }
 	},
 
-	{ "NUL in header name", NULL,
-	  "HTTP/1.1 200 OK\r\nF\x00oo: bar\r\n", 28,
-	  -1, 0, NULL,
-	  { { NULL } }
-	},
-
-	{ "NUL in header value", NULL,
-	  "HTTP/1.1 200 OK\r\nFoo: b\x00ar\r\n", 28,
-	  -1, 0, NULL,
-	  { { NULL } }
-	},
-
 	/* Failing test from Cockpit */
 
 	{ "Partial response stops after HTTP/", NULL,
@@ -900,6 +908,17 @@ do_qvalue_tests (void)
 
 		debug_printf (1, "    acceptable: ");
 		if (acceptable) {
+			/* Kludge to deal with the fact that the sort order of the first
+			 * test is not fully specified.
+			 */
+			if (i == 0 && acceptable->next &&
+			    !g_str_equal (acceptable->data, qvaluetests[i].acceptable[0]) &&
+			    g_str_equal (acceptable->data, qvaluetests[i].acceptable[1])) {
+				gpointer tmp = acceptable->data;
+				acceptable->data = acceptable->next->data;
+				acceptable->next->data = tmp;
+			}
+
 			for (iter = acceptable, j = 0; iter; iter = iter->next, j++) {
 				debug_printf (1, "%s ", (char *)iter->data);
 				g_assert_cmpstr (iter->data, ==, qvaluetests[i].acceptable[j]);
